@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <limits.h>
+#include <ctype.h>
 
 #include <errno.h>
 #include <gdk/gdkkeysyms.h>
@@ -43,6 +44,7 @@
 #include "misc.h"
 #include "menu.h"
 #include "menu_callbacks.h"
+#include "glade_main_callbacks.h"
 #include "callbacks.h"
 #include "dnd.h"
 #include "entry.h"
@@ -62,6 +64,9 @@
 #include "remove.h"
 #include "callbacks.h"
 
+#define GLADE_HOOKUP_OBJECT(component,widget,name) \
+  g_object_set_data_full (G_OBJECT (component), name, \
+    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)
 
 typedef struct treefind_t {
 	GtkTreeView *treeview;
@@ -75,34 +80,10 @@ typedef struct treefind_t {
 	gboolean done;
 } treefind_t;
 
+/* XXX: (cleanup) these variables should be treestuff */
 static GtkTreeRowReference *Rref=NULL,*Lref=NULL;
 extern gboolean easy_mode;
 
-static void show_alt_treeview(GtkWidget * parent)
-{
-    GtkWidget *vpaned;
-    if(!parent)
-	return;
-    /*printf("DBG:show_text\n");*/
-    vpaned = WIDGET("hpaned1");
-    if (gtk_paned_get_position((GtkPaned *)vpaned) < vpaned->allocation.height * 0.40)
-       gtk_paned_set_position(GTK_PANED(vpaned), vpaned->allocation.height * 0.40);
-    if (strstr(tree_details->argv[0],"xffm")){
-        tree_details->hpane = gtk_paned_get_position((GtkPaned *)vpaned);
-	tree_details->hpane_ratio = (double)tree_details->hpane / (double)tree_details->geometryX;
-    }
-       
-}
-static void hide_alt_treeview(GtkWidget * parent)
-{
-    GtkWidget *vpaned;
-    if(!parent)
-	return;
-    /*printf("DBG:show_text\n");*/
-    vpaned = WIDGET("hpaned1");
-    gtk_paned_set_position(GTK_PANED(vpaned), 0);
-}
-
 static void status_the_path(GtkTreeView *treeview, GtkTreePath *ipath,tree_entry_t *en)
 	    {
 		gchar *readable_path;
@@ -122,9 +103,8 @@ static void status_the_path(GtkTreeView *treeview, GtkTreePath *ipath,tree_entry
 
 G_MODULE_EXPORT
 void set_path_reference(GtkTreeView *treeview,GtkTreePath *ipath){
-#if 0
     	GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-	if (tree_details->treeview==treeview){
+	if (get_tree_id (treeview) == 0){
 		if (Rref) gtk_tree_row_reference_free(Rref);
 		if (!ipath) Rref=NULL;
 		else Rref= gtk_tree_row_reference_new(treemodel, ipath);
@@ -133,7 +113,6 @@ void set_path_reference(GtkTreeView *treeview,GtkTreePath *ipath){
 		if (!ipath) Lref=NULL;
 		else Lref= gtk_tree_row_reference_new(treemodel, ipath);
 	}
-#endif
 }
 
 static gboolean find_next_row(GtkTreeModel * treemodel, GtkTreePath * treepath, GtkTreeIter * iter, gpointer data)
@@ -145,9 +124,7 @@ static gboolean find_next_row(GtkTreeModel * treemodel, GtkTreePath * treepath,
 
     if (treefind->done) return FALSE;
     if (!treepath || !treefind->ipath) {
-#ifdef DEBUG
-	    printf("DBG:bailing out 1, keybindings.c\n");
-#endif
+	    TRACE("bailing out 1, keybindings.c\n");
 	    treefind->done=TRUE;
 	    return TRUE;
     }
@@ -166,9 +143,7 @@ static gboolean find_next_row(GtkTreeModel * treemodel, GtkTreePath * treepath,
     }
     gtk_tree_model_get(treemodel, iter, NAME_COLUMN, &f, ENTRY_COLUMN, &en, -1);
     if (!f || !strlen(f)) {
-#ifdef DEBUG	
-	printf("DBG: treemodel_get returns nothing for name column cell!\n");
-#endif			
+	TRACE("treemodel_get returns nothing for name column cell!\n");
 	if (f) g_free(f);
 	return FALSE;
     }
@@ -202,9 +177,7 @@ static gboolean find_prev_row(GtkTreeModel * treemodel, GtkTreePath * treepath,
 
     if (treefind->done) return FALSE;
     if (!treepath || !treefind->ipath) {
-#ifdef DEBUG
-	    printf("DBG:bailing out 2, keybindings.c\n");
-#endif
+	    TRACE("bailing out 2, keybindings.c\n");
 	    treefind->done=TRUE;
 	    return TRUE;
     }
@@ -212,9 +185,7 @@ static gboolean find_prev_row(GtkTreeModel * treemodel, GtkTreePath * treepath,
     if (gtk_tree_path_compare (treepath,treefind->ipath) > 0) return FALSE;
     if (gtk_tree_path_compare (treepath,treefind->ipath) == 0) {
 	    if (treefind->prev_path){
-#ifdef DEBUG
-		    printf("DBG:previous path found=%s\n",treefind->en->path);
-#endif
+		TRACE("previous path found=%s\n",treefind->en->path);
         	gdk_flush();
     		gtk_tree_view_scroll_to_cell(treefind->treeview, treefind->prev_path, NULL, TRUE, 0.0, 0.0);
 		clear_dnd_selection_list();
@@ -242,9 +213,7 @@ static gboolean find_prev_row(GtkTreeModel * treemodel, GtkTreePath * treepath,
     }
     gtk_tree_model_get(treemodel, iter, NAME_COLUMN, &f, ENTRY_COLUMN, &en, -1);
     if (!f || !strlen(f)) {
-#ifdef DEBUG	
-	printf("DBG: treemodel_get returns nothing for name column cell!\n");
-#endif		
+	TRACE("treemodel_get returns nothing for name column cell!\n");
 	if (f) g_free(f);	
 	return FALSE;
     }
@@ -255,21 +224,71 @@ static gboolean find_prev_row(GtkTreeModel * treemodel, GtkTreePath * treepath,
         if (treefind->prev_path) gtk_tree_path_free(treefind->prev_path);
 	treefind->prev_path=gtk_tree_path_copy(treepath);
         gtk_tree_model_get(treemodel, iter,ENTRY_COLUMN, &(treefind->en), -1);
-#ifdef DEBUG
-	printf("DBG:found %c==%c==%c\n",*f,treefind->C,treefind->c);
-	printf("DBG:treefind->en->path=%s\n",treefind->en->path);
-#endif
+	TRACE("found %c==%c==%c\n",*f,treefind->C,treefind->c);
+	TRACE("treefind->en->path=%s\n",treefind->en->path);
 	g_free(ff);
     	return FALSE;
     } 
     g_free(ff);
-#ifdef DEBUG
-	printf("DBG:returning FALSE\n");
-#endif
+	TRACE("returning FALSE\n");
 
     return FALSE;
 }
 
+static void change_to_treeview(int tree_id){
+    GtkTreePath *ipath=NULL;
+    GtkTreeRowReference **ref;
+    if (tree_id==0) {
+		on_unselect_activate((GtkMenuItem *)tree_details->treestuff[1].treeview,NULL);
+	ref=&Rref;
+    }
+    else {
+		on_unselect_activate((GtkMenuItem *)tree_details->treestuff[0].treeview,NULL);
+	ref=&Lref;
+    }
+    if (*ref && gtk_tree_row_reference_valid(*ref)) {
+	ipath=gtk_tree_row_reference_get_path(*ref);	
+    } else {
+    if (*ref) gtk_tree_row_reference_free(*ref);
+	ipath=gtk_tree_path_new_first();
+	*ref=gtk_tree_row_reference_new(tree_details->treestuff[tree_id].treemodel, ipath);
+    }
+    gtk_widget_grab_focus((GtkWidget *) tree_details->treestuff[tree_id].treeview);
+    gtk_tree_view_set_cursor (tree_details->treestuff[tree_id].treeview,ipath,NULL,FALSE);
+    gtk_tree_selection_select_path(tree_details->treestuff[tree_id].selection,ipath);
+    if(ipath) gtk_tree_path_free(ipath);
+}
+
+G_MODULE_EXPORT
+void
+on_pane1_clicked                       (GtkButton       *button,
+                                        gpointer         user_data){
+    GtkWidget *hpaned=WIDGET("hpaned1");
+    tree_details->hpane=hpaned->allocation.width;
+    change_to_treeview(1);
+    gtk_paned_set_position(GTK_PANED(hpaned), tree_details->hpane);
+    tree_details->hpane_ratio=1.0;
+}
+
+G_MODULE_EXPORT
+void
+on_pane0_clicked                       (GtkButton       *button,
+                                        gpointer         user_data){
+    GtkWidget *hpaned=WIDGET("hpaned1");
+    tree_details->hpane=0;
+    change_to_treeview(0);
+    gtk_paned_set_position(GTK_PANED(hpaned), tree_details->hpane);
+    tree_details->hpane_ratio=0.0;
+}
+G_MODULE_EXPORT
+void
+on_pane10_clicked                       (GtkButton       *button,
+                                        gpointer         user_data){
+    GtkWidget *hpaned=WIDGET("hpaned1");
+    tree_details->hpane=hpaned->allocation.width/2;
+    gtk_paned_set_position(GTK_PANED(hpaned), tree_details->hpane);
+    tree_details->hpane_ratio=0.50;
+}
 
 
 /* additional key bindings for the tree */
@@ -279,28 +298,77 @@ gboolean treeview_key(GtkWidget *w,
 	GtkTreeView *treeview=(GtkTreeView *)w;
     	GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     	GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-	    GtkTreePath *ipath=NULL;
+	GtkTreePath *ipath=NULL;
 	GtkTreeIter iter;
 	static gboolean lock=FALSE;
         tree_entry_t *en=NULL;
 	/*treestuff_t *treestuff=get_treestuff(treeview);*/
 
-    if (event->state&(GDK_CONTROL_MASK)) return FALSE;
+    if (event->state&(GDK_CONTROL_MASK)){
+	/* inactive menu item accels should be caught here 
+	 * this is a workaround for gtk accelerators to hidden
+	 * menu element not working (they used to, but not any
+	 * more)*/
+	
+	switch (toupper(event->keyval)){
+	    case GDK_I:
+		on_properties_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_C:
+		on_copy_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_V:
+		on_paste_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_X:
+		on_cut_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_Q:
+		on_close_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_Y:
+		on_countfiles_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_Z:
+		on_refresh_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_B:
+		on_open_book_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_A:
+		on_select_all_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_P:
+		on_print_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_D:
+		on_duplicate_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_S:
+		on_symlink_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_R:
+		on_rename_activate((GtkMenuItem *)w,NULL);
+		break;
+	    case GDK_M:
+		on_touch_activate((GtkMenuItem *)w,NULL);
+		break;
+	    default:
+		break;
+	}
+	return FALSE;
+    }
    /* asian input methods */
-    if (event->keyval == GDK_space &&(event->state&(GDK_MOD1_MASK|GDK_CONTROL_MASK|GDK_SHIFT_MASK))){
+    if (event->keyval == GDK_space &&(event->state&(GDK_MOD1_MASK|GDK_SHIFT_MASK))){
 	return FALSE;
     }
 	easy_mode = FALSE; /*may be better down below */
 	if (tree_details->loading){
-#ifdef DEBUG	
-			printf("DBG: ignoring key...\n");
-#endif
+			TRACE("ignoring key...\n");
 		return TRUE;
 	}
 	if (lock) {
-#ifdef DEBUG	
-		printf("DBG: keys locked!\n");
-#endif			
+		TRACE("keys locked!\n");
 		return TRUE;
 	}
 	{ 
@@ -311,10 +379,8 @@ gboolean treeview_key(GtkWidget *w,
 	}
 
 	hide_text(w);
-#ifdef DEBUG	
-	printf("DBG: keyval=0x%x state=0x%x, mods= 0x%x 0x%x 0x%x 0x%x 0x%x \n",
+	TRACE("keyval=0x%x state=0x%x, mods= 0x%x 0x%x 0x%x 0x%x 0x%x \n",
 			event->keyval,event->state,GDK_MOD1_MASK,GDK_MOD2_MASK,GDK_MOD3_MASK,GDK_MOD4_MASK,GDK_MOD5_MASK);
-#endif
 	switch (event->keyval){
 		case GDK_Control_L:
 		case GDK_Control_R:
@@ -350,103 +416,110 @@ gboolean treeview_key(GtkWidget *w,
 			on_remove_activate((GtkMenuItem *)w, NULL);
 			return TRUE;
 
-#ifdef FIXME
 		case GDK_Tab:
 		{
-			GtkTreeRowReference **ref;
-    			GtkTreeSelection *selection2; 
-			GtkTreeView *treeview2;
+			
+			if (get_only_visible_treestuff()) return TRUE;
+			    
 			lock=TRUE;
-			on_unselect_activate((GtkMenuItem *)treeview,NULL);
-			if ((event->state&GDK_MOD5_MASK)){
-				treeview2=tree_details->treeview;
-				ref=&Rref;
-				hide_alt_treeview((GtkWidget *)treeview);
-			} else {	
-			  if (tree_details->treeview==treeview){
-				ref=&Lref;
-				treeview2=(GtkTreeView *)WIDGET("treeview2");
-				show_alt_treeview((GtkWidget *)treeview);
-			  } else {
-				treeview2=tree_details->treeview;
-				ref=&Rref;
-			  }
-			}
-			selection2 = gtk_tree_view_get_selection(treeview2);
-
-
-
-			
-			if (*ref && gtk_tree_row_reference_valid(*ref)) {
-				ipath=gtk_tree_row_reference_get_path(*ref);
-				
+			if (get_tree_id (treeview) == 0){
+			    change_to_treeview(1);	
 			} else {
-				if (*ref) gtk_tree_row_reference_free(*ref);
-				ipath=gtk_tree_path_new_first();
-				*ref=gtk_tree_row_reference_new(treemodel, ipath);
+			    change_to_treeview(0);	
 			}
+			    
 			
-					
-			gtk_widget_grab_focus((GtkWidget *) treeview2);
-    		  	gtk_tree_view_set_cursor (treeview2,ipath,NULL,FALSE);
-			gtk_tree_selection_select_path(selection2,ipath);
-			
-			gtk_tree_model_get_iter(treemodel, &iter, ipath);
-			gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
-
-			if (en) status_the_path(treeview,ipath,en);
-			
-			gtk_tree_path_free(ipath);
 		        lock=FALSE;
 			turn_on();
  
 			return TRUE;
 		}
-#endif /*FIXME*/
 		case GDK_F10:
 		{	 
 			do_main_popup("main_menu2_menu",0);
 			return TRUE;
 		}
-		case GDK_F2:
+		case GDK_F1:
+		{	 
+		    static GtkWidget *help_menu = NULL;
+		    if (!help_menu) {
+			GtkWidget *menuitem;
+			gchar *g,**p,*t[]={"F1","Help",
+				    "F2","Sort",
+				    "F3","Tools",
+				    "F4","New window",
+				    "F5","Go",
+				    "F6","Options",
+				    "F7","Preferences",
+				    "F8","File",
+				    "F9","Actions",/*popup*/
+				    "F10","Main menu",
+				    "F11","Show left pane",
+				    "F12","Show right pane",
+				    NULL,NULL
+			};
+			help_menu = gtk_menu_new ();
+			for (p=t; *p; p+=2){
+			    g=g_strdup_printf("%s -- %s",*p,_(*(p+1)));
+			    menuitem = gtk_menu_item_new_with_mnemonic (g);
+			    g_free(g);
+			    gtk_widget_show (menuitem);
+			   /* gtk_widget_set_sensitive(menuitem, FALSE);*/
+			    gtk_container_add (GTK_CONTAINER (help_menu), menuitem);
+			}
+			GLADE_HOOKUP_OBJECT (tree_details->window, help_menu,"help_menu");
+		    }
+		    do_main_popup("help_menu",0);
+		    return TRUE;
+		}
+		case GDK_F3:
 		{	 
 			do_main_popup("tools1_menu",0);
 			return TRUE;
 		}
-		case GDK_F3:
+		case GDK_F4:
 		{	 
 			do_main_popup("open1_menu",0);
 			return TRUE;
 		}
-		case GDK_F4:
+		case GDK_F5:
 		{	 
 			do_main_popup("go1_menu",0);
 			return TRUE;
 		}
-		case GDK_F5:
-		{	 
-			do_main_popup("limpiar1_menu",0);
-			return TRUE;
-		}
 		case GDK_F6:
-		{	 
-			/*do_main_popup("themes1_menu",0);*/
-			return TRUE;
-		}
-		case GDK_F7 :
 		{	 
 			do_main_popup("options1_menu",0);
 			return TRUE;
 		}
-		case GDK_F8 :
+		case GDK_F7:
 		{	 
 			do_main_popup("preferences3_menu",0);
 			return TRUE;
 		}
+		case GDK_F11 : /* hide show treeview 1 */
+		{	 
+			lock=TRUE;
+		   if (get_only_visible_treestuff()) on_pane10_clicked (NULL,NULL);
+		   else {
+		       on_pane1_clicked (NULL,NULL);
+		   }
+			lock=FALSE;
+			return TRUE;
+		}
+		case GDK_F12 : /* hide show treeview 1 */
+		{
+			lock=TRUE;
+		   if (get_only_visible_treestuff()) on_pane10_clicked (NULL,NULL);
+		   else {
+			on_pane0_clicked (NULL,NULL);
+		   }
+			lock=FALSE;
+			return TRUE;
+		}
 		case GDK_Menu:
-		case GDK_F12:
-		case GDK_F11:
-		case GDK_F1:
+		case GDK_F8:
+		case GDK_F2:
 		case GDK_F9:
 		{	 
 			GtkTreePath *selectpath=NULL;
@@ -469,12 +542,7 @@ gboolean treeview_key(GtkWidget *w,
 			  if (en) selectpath=gtk_tree_model_get_path (treemodel,&iter);
 			}
 			switch (event->keyval){
-			  case GDK_F11:
-			    /*if (tree_details->selectionOK==1) set_sense(0,TRUE);
-			    else set_sense(0,FALSE);
-			    do_main_popup("book1_menu",0);*/
-			    break;
-			  case GDK_F1:
+			  case GDK_F2:
 			    if (tree_details->selectionOK==1) {
 				set_sense(2,TRUE);
 			        if(!en || IS_BROKEN_LNK(en->type)) set_sense(2,FALSE);
@@ -482,23 +550,23 @@ gboolean treeview_key(GtkWidget *w,
 			    else set_sense(2,FALSE);
 			    do_main_popup("sort1_menu",0);
 			    break;
-			  case GDK_F12:
+			  case GDK_F8:
 			    if (tree_details->selectionOK==1 && IS_PATH(en->type)) {
 				set_sense(1,TRUE);
 			        if(!en || IS_BROKEN_LNK(en->type)) set_sense(1,FALSE);
 				/* this block is repeated in menu.c */
 				if (IS_DIR(en->type)|| !IS_FILE(en->type)){
-				    gtk_widget_set_sensitive(WIDGET("scramble1"),FALSE);
-				    gtk_widget_set_sensitive(WIDGET("unscramble1"),FALSE);
+				    gtk_widget_set_sensitive(WIDGET("sb_scramble_menuitem"),FALSE);
+				    gtk_widget_set_sensitive(WIDGET("sb_unscramble_menuitem"),FALSE);
 				} else {
 				    gchar *p=strrchr(en->path,'.');
 				    if (!p || strcmp(p,".cyt")!=0) {
-				      gtk_widget_set_sensitive(WIDGET("scramble1"),TRUE);
-				      gtk_widget_set_sensitive(WIDGET("unscramble1"),FALSE);
+				      gtk_widget_set_sensitive(WIDGET("sb_scramble_menuitem"),TRUE);
+				      gtk_widget_set_sensitive(WIDGET("sb_unscramble_menuitem"),FALSE);
 				    }
 				    else {
-				      gtk_widget_set_sensitive(WIDGET("unscramble1"),TRUE);
-				      gtk_widget_set_sensitive(WIDGET("scramble1"),FALSE);
+				      gtk_widget_set_sensitive(WIDGET("sb_unscramble_menuitem"),TRUE);
+				      gtk_widget_set_sensitive(WIDGET("sb_scramble_menuitem"),FALSE);
 				    }
 				}
 				/* end of repeated block */
@@ -569,15 +637,11 @@ gboolean treeview_key(GtkWidget *w,
 	    if (event->state & GDK_SHIFT_MASK){
 		    backwards = TRUE;
 		    treefind.second_letter_match=FALSE;
-#ifdef DEBUG	
-		    printf("DBG: going backwards!\n");
-#endif			
+		    TRACE("going backwards!\n");
 	    }
 
 
-#ifdef DEBUG	
-	    printf("DBG: alphabeta keyval=0x%x\n",event->keyval);
-#endif			
+	    TRACE("alphabeta keyval=0x%x\n",event->keyval);
 	    /*if (!en) {
 		if (!gtk_tree_model_get_iter_first (treemodel,&iter))
 		    return FALSE;
@@ -602,7 +666,10 @@ gboolean treeview_key(GtkWidget *w,
 							"\n",NULL);
 	    }
 
-    	    if (treefind.ipath) {gtk_tree_path_free(treefind.ipath);ipath=NULL;}
+    	    if (treefind.ipath) {
+		gtk_tree_path_free(treefind.ipath);
+		treefind.ipath=NULL;
+	    }
 	    return TRUE;
 	}
 	
@@ -614,24 +681,27 @@ gboolean treeview_key(GtkWidget *w,
 	        case GDK_Find:
 			 sb_find(w);
 			 return TRUE;
+		case GDK_KP_Down:
 		case GDK_Down:
-#ifdef DEBUG
-		     printf("DBG:gdk down...state=0x%x, mod3 is 0x%x\n",
-				     event->state,GDK_MOD3_MASK);
-#endif
+		     TRACE("gdk down...state=0x%x, mod3 is 0x%x\n", event->state,GDK_MOD3_MASK);
+	        case GDK_KP_Up:
 	        case GDK_Up:
+#if 0
+		     /* default up/down is more limited than the replacement code */
+		     return FALSE;
+#else
 			if (event->state&GDK_SHIFT_MASK) return FALSE;
 		         lock=TRUE;
-#ifdef DEBUG	
-		         printf("DBG: lock #1!\n");
-#endif			
+		         TRACE("lock #1!\n");
 	    		if (!en) {
 			  if (!gtk_tree_model_get_iter_first (treemodel,&iter))
 		          lock=FALSE;
 		    	  return TRUE;
 	    		} 
+			if (ipath) gtk_tree_path_free(ipath);
 	    		ipath=gtk_tree_model_get_path(treemodel, &iter);
-			if (event->keyval==GDK_Down) {
+			if (!ipath) return TRUE;
+			if (event->keyval==GDK_Down || event->keyval==GDK_KP_Down) {
 				GtkTreeIter iter2;
 				if (gtk_tree_view_row_expanded (treeview,ipath)){
 					gtk_tree_path_down (ipath);
@@ -639,134 +709,165 @@ gboolean treeview_key(GtkWidget *w,
 					gtk_tree_path_next (ipath);
 				}
 				if (!gtk_tree_model_get_iter (treemodel,&iter2,ipath)){
-					if (!gtk_tree_model_iter_next(treemodel,&iter)){
-				  	   gtk_tree_path_free(ipath);
-				  	   lock=FALSE;
-		    	  	  	   return TRUE;
+					iter2=iter;
+					if (!gtk_tree_model_iter_next(treemodel,&iter))
+					{
+					   gboolean has_parent;
+					   while ((has_parent=gtk_tree_model_iter_parent(treemodel,&iter,&iter2))!=FALSE)
+					   {
+					       iter2=iter;
+					       if (gtk_tree_model_iter_next(treemodel,&iter)) 
+					       {
+						   iter2=iter;
+						   break;
+					       }
+					   }
+					   if (has_parent)
+					   {
+					    if (ipath) gtk_tree_path_free(ipath);
+					    ipath=gtk_tree_model_get_path(treemodel, &iter2);
+		    			    gtk_tree_path_next (ipath);
+					   }
+					   else {
+				  	     gtk_tree_path_free(ipath);
+					     ipath=NULL;
+				  	     lock=FALSE;
+		    	  	  	     return TRUE;
+					   }
+					   
 					}
+					if (ipath) gtk_tree_path_free(ipath);
 	    				ipath=gtk_tree_model_get_path(treemodel, &iter);
 				}
 			}
-			if (event->keyval==GDK_Up) {
+			if (event->keyval==GDK_Up || event->keyval==GDK_KP_Up) {
+				GtkTreeIter iter2;
 				GtkTreePath *rpath=gtk_tree_path_new_first();
-				if (!rpath || !ipath || gtk_tree_path_compare (rpath,ipath)==0){
+				if (!rpath || gtk_tree_path_compare (rpath,ipath)==0){
 				  if (ipath) gtk_tree_path_free(ipath);
-				  ipath=NULL;
 				  if (rpath) gtk_tree_path_free(rpath);
 				  lock=FALSE;
 		    	  	  return TRUE;
 				}
 				gtk_tree_path_free(rpath);
-				if (!gtk_tree_path_prev (ipath)) gtk_tree_path_up(ipath);
+				if (!gtk_tree_path_prev (ipath)) 
+				{
+				    gtk_tree_path_up(ipath);
+				} else 
+				{
+				    while (gtk_tree_view_row_expanded (treeview,ipath)){
+					gtk_tree_path_down (ipath);			
+					gtk_tree_model_get_iter (treemodel,&iter2,ipath);
+					iter=iter2;
+					while (1) {
+					    if (!gtk_tree_model_iter_next(treemodel,&iter)){
+						iter=iter2; 
+						break;
+					    }
+					    iter2=iter;
+					}
+					if (ipath) gtk_tree_path_free(ipath);
+					ipath=gtk_tree_model_get_path(treemodel, &iter);
+				    }
+				    
+				}
 			}
 			
 			set_path_reference(treeview,ipath);
 		    	gtk_widget_grab_focus((GtkWidget *) treeview);
-    		  	gtk_tree_view_set_cursor (treeview,ipath,NULL,FALSE);
-			gtk_tree_selection_select_path(selection,ipath);
-			
-			gtk_tree_model_get_iter(treemodel, &iter, ipath);
-			gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
+			if (ipath){
+			    gtk_tree_view_set_cursor (treeview,ipath,NULL,FALSE);
+			    gtk_tree_selection_select_path(selection,ipath);
+			    gtk_tree_model_get_iter(treemodel, &iter, ipath);
+			    gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
 
-			if (en) status_the_path(treeview,ipath,en);
-			/* scrolling with alt key */	
-			if (event->state&GDK_MOD1_MASK){
+			    if (en) status_the_path(treeview,ipath,en);
+			    /* scrolling with alt key */	
+			    if (event->state&GDK_MOD1_MASK){
 				/*get selected path ++*/
 			     gdk_flush();
 			     gtk_tree_view_scroll_to_cell(treeview, ipath, NULL, TRUE, 0.0, 0.0);
 			     clear_dnd_selection_list();
 			     get_dnd_selection(treemodel, ipath, &iter, (gpointer) treeview);
-			}
+			    }
 			
-			gtk_tree_path_free(ipath);
+			    gtk_tree_path_free(ipath);
+			}
 		        lock=FALSE;
-#ifdef DEBUG	
-			printf("DBG: unlock #1!\n");
-#endif			
+			TRACE("unlock #1!\n");
 		    	return TRUE;
+#endif
 		case GDK_Right:
+		case GDK_KP_Right:
 		{
 			lock=TRUE;
-#ifdef DEBUG	
-			printf("DBG: lock #2...\n");
-#endif			
+			TRACE("lock #2...\n");
+			if (ipath) gtk_tree_path_free(ipath);
+			ipath = gtk_tree_model_get_path(treemodel, &iter);
 			if (gtk_tree_model_iter_has_child(treemodel,&iter))
 			{
-    			     GtkTreePath *treepath;
-#ifdef DEBUG	
-			printf("DBG: lock #2a...path=%s\n",(en->path)?en->path:"null");
-#endif			
-			     treepath = gtk_tree_model_get_path(treemodel, &iter);
-			     gtk_tree_view_expand_row(treeview, treepath, FALSE);
-		    	     gtk_tree_path_free(treepath);
+			     TRACE("lock #2a...path=%s\n",(en->path)?en->path:"null");
+			     gtk_tree_view_expand_row(treeview, ipath, FALSE);
+			     gtk_tree_selection_select_path (selection,ipath);
+			     gtk_tree_view_set_cursor (treeview,ipath,NULL,FALSE);
+		             get_dnd_selection(treemodel, ipath, &iter, (gpointer) treeview);			
+			     set_path_reference(treeview,ipath);
 			     turn_on();
 			     lock=FALSE;
-#ifdef DEBUG	
-			     printf("DBG: unlock #2a!\n");
-#endif			
-			     return TRUE;
+			     TRACE("unlock #2a!\n");
+			      if (ipath) gtk_tree_path_free(ipath);
+			      ipath=NULL;
+			      return TRUE;
 			}
-#if 0
-/* better leave this to the return key only */			
-			if (IS_LOCAL_TYPE(en->type) && IS_DIR(en->type)){
-#ifdef DEBUG	
-			printf("DBG: lock #2aa...path=%s\n",(en->path)?en->path:"null");
-#endif			
-			     go_to(treeview, en->path);
-			     turn_on();
-#ifdef DEBUG	
-		 	     printf("DBG: unlock #2aa!\n");
-#endif			
-			     lock=FALSE;
-			     return TRUE;
-			}
-#endif
 			/* otherwise, -> is a return: */
 		}
 		case GDK_Return:
 			 lock=TRUE;
-#ifdef DEBUG	
-			 printf("DBG: lock #2b!\n");
-#endif			
+			 TRACE("lock #2b!\n");
 			 on_double_click_activate((GtkMenuItem *)w,
 					(gpointer) treeview);
 			 lock=FALSE;
-#ifdef DEBUG	
-			 printf("DBG: unlock #2b!\n");
-#endif			
+			 TRACE("unlock #2b!\n");
 			 return TRUE;
 			 break;
+		case GDK_KP_Left:
 		case GDK_Left:
 			lock=TRUE;
-#ifdef DEBUG	
-			printf("DBG: lock #3!\n");
-#endif			
+			TRACE("lock #3!\n");
 			
 			{	
 			  GtkTreeIter parent;
-			  GtkTreePath *tpath=gtk_tree_model_get_path(treemodel, &iter);
+			  /*if (ipath) gtk_tree_path_free(ipath);*/
+			  ipath = gtk_tree_model_get_path(treemodel, &iter);
 			  
-			  if (gtk_tree_view_row_expanded (treeview,tpath)){
-			     gtk_tree_view_collapse_row(treeview, tpath);
-		    	     gtk_tree_path_free(tpath);
+			  if (gtk_tree_view_row_expanded (treeview,ipath)){
+			     gtk_tree_view_collapse_row(treeview, ipath);
 			     turn_on();
+			     gtk_tree_selection_select_path (selection,ipath);
+			     gtk_tree_view_set_cursor (treeview,ipath,NULL,FALSE);
+		             get_dnd_selection(treemodel, ipath, &iter, (gpointer) treeview);
+			     set_path_reference(treeview,ipath);
+		    	     gtk_tree_path_free(ipath);
+			     ipath=NULL;
 			     lock=FALSE;
 			     return TRUE;
 			  } 
 			  if (gtk_tree_model_iter_parent (treemodel,&parent,&iter)){
-		    	     gtk_tree_path_free(tpath);
-			     tpath=gtk_tree_model_get_path(treemodel, &parent);
-			     gtk_tree_view_collapse_row(treeview, tpath);
+			     if (ipath) gtk_tree_path_free(ipath);
+			     ipath=gtk_tree_model_get_path(treemodel, &parent);
+			     gtk_tree_view_collapse_row(treeview, ipath);
 				     
-    			     gtk_tree_view_scroll_to_cell(treeview, tpath, NULL, TRUE, 0.0, 0.0);
+    			     gtk_tree_view_scroll_to_cell(treeview, ipath, NULL, TRUE, 0.0, 0.0);
 
 			     clear_dnd_selection_list();
 
-			     gtk_tree_selection_select_path (selection,tpath);
-			     gtk_tree_view_set_cursor (treeview,tpath,NULL,FALSE);
-		             get_dnd_selection(treemodel, tpath, &parent, (gpointer) treeview);
+			     gtk_tree_selection_select_path (selection,ipath);
+			     gtk_tree_view_set_cursor (treeview,ipath,NULL,FALSE);
+		             get_dnd_selection(treemodel, ipath, &parent, (gpointer) treeview);
+			     set_path_reference(treeview,ipath);
 
-		    	     gtk_tree_path_free(tpath);
+		    	     gtk_tree_path_free(ipath);
+			     ipath=NULL;
 			     turn_on();
 			     lock=FALSE;
 			     return TRUE;
@@ -775,9 +876,7 @@ gboolean treeview_key(GtkWidget *w,
 			
 			
 			lock=FALSE;
-#ifdef DEBUG	
-			printf("DBG: unlock #3!\n");
-#endif			
+			TRACE("unlock #3!\n");
 			return TRUE;
 			
 			break;
